/*
   PCX.C
   A set of functions to read a PCX file,

   08/23/93
*/


#include <stdio.h>
#include <alloc.h>
#include <dos.h>
#include <conio.h>
#include <string.h>

#include "pcx.h"



/*_________________________________________________________________________
 |                                                                         |
 | PcxLoad()                                                               |
 | Read in a pcx file and load into memory.                                |
 |_________________________________________________________________________|*/
int PcxLoad(char far *filename, PCX *pcx)
{
FILE *infile;

   /* Open the PCX file */
   if ((infile=fopen(filename, "rb"))==NULL)
      return 0;

   /* initialize the file name element */
   strcpy(pcx->filename, filename);

   /* Read the file header */
   PcxLoadHeader(infile, pcx);

   /* Read the image data */
   PcxLoadImage(infile, pcx);

   /* read the palette data */
   PcxLoadPalette(infile, pcx);

   /* close PCX file */
   fclose(infile);
   return 1;
}



/*_________________________________________________________________________
 |                                                                         |
 | PcxLoadImage()                                                          |
 | Grab image from pcx file and stores it in image buffer.                 |
 |_________________________________________________________________________|*/
void PcxLoadImage(FILE *pcxfile, PCX *pcx)
{
const int BYTEMODE=0, RUNMODE=1;
int mode=BYTEMODE;
static unsigned char outbyte, bytecount;
long pcxwidth, pcxheight;
long i;


   /* get the size of the image stored */
   pcxwidth  = pcx->header.xMax - pcx->header.xMin + 1;
   pcxheight = pcx->header.yMax - pcx->header.yMin + 1;
   pcx->imageSize=pcxwidth*pcxheight;

   /* create storage for image */
   pcx->image = (char *) malloc(pcx->imageSize);

   /* skip past header */
   fseek(pcxfile, 128L, SEEK_SET);

   /* loop through all bytes of image */
   for (i=0; i < pcx->imageSize; i++)
      {
      /* see if we need to read a byte */
      if (mode==BYTEMODE)
         {
         /* get byte */
         outbyte=(unsigned char)fgetc(pcxfile);
         /* see if byte says we have a run */
         if (outbyte>0xbf)
            {
            /*
             * get the number of bytes in the run
             * (maximum run lenght is 63 [0x3f])
             * so we strip the count from the byte
             */
            bytecount = (int)((int)outbyte & 0x3f);
            /* get the color byte that is set for the run */
            outbyte=(unsigned char)fgetc(pcxfile);
            /* if we still have bytes in the run */
            if (--bytecount > 0)
               /* keep us in run length mode */
               mode = RUNMODE;
            }
         }
      /*
       * while in run length mode
       * decrement the byte counter
       * until we are empty
       */
      else if (--bytecount == 0)
         /* then reset to byte mode */
         mode=BYTEMODE;
      /* store the byte in our image array */
      pcx->image[i]=outbyte;
      }
}





/*_________________________________________________________________________
 |                                                                         |
 | PcxLoadHeader()                                                         |
 | Grab header data from pcx file and store in header_struct.              |
 |_________________________________________________________________________|*/
void PcxLoadHeader(FILE *pcxfile, PCX *pcx)
{
   fread((char *)&pcx->header, sizeof(pcx_header), 1, pcxfile);
}


/*_________________________________________________________________________
 |                                                                         |
 | PcxLoadPalette()                                                        |
 | Grab palette from pcx file and store in palette buffer.                 |
 |_________________________________________________________________________|*/
void PcxLoadPalette(FILE *pcxfile, PCX *pcx)
{
int i, j;

   fseek(pcxfile, -768L, SEEK_END);
   for (i=0; i<256; i++)
      for (j=0; j<3; j++)
         pcx->palette[i*3+j]=fgetc(pcxfile)/*>>2*/;
}

/*_________________________________________________________________________
 |                                                                         |
 | PcxDisplayHeader()                                                      | 
 | Displays header information contained in the header_struct.             |
 |_________________________________________________________________________|*/
void PcxDisplayHeader(PCX *pcx)
{
   printf("\n\n Contents of the PCX file header");
   printf("\n for %s:",pcx->filename);
   printf("\n --------------------------------");
   printf("\n PCX Manufacturer ID is   : %d",pcx->header.manufacturer);
   printf("\n PCX version is           : %d",pcx->header.version);
   printf("\n PCX enoding method is    : %d",pcx->header.encoding);
   printf("\n Bits per pixel are       : %d",pcx->header.bits_per_pixel);
   printf("\n Starting X is            : %d",pcx->header.xMin);
   printf("\n Starting Y is            : %d",pcx->header.yMin);
   printf("\n Horizontal size is       : %d",pcx->header.xMax);
   printf("\n Vertical size is         : %d",pcx->header.yMax);
   printf("\n Horizontal resolution is : %d",pcx->header.hres);
   printf("\n Vertical resolution is   : %d",pcx->header.vres);
   printf("\n EGA palette is           : %s",pcx->header.palette16);
   printf("\n RESERVED                   %c",pcx->header.reserved);
   printf("\n Number of bit planes is  : %d",pcx->header.color_planes);
   printf("\n Number of bytes per row  : %d",pcx->header.bytes_per_line);
   printf("\n Palette information is   : %d",pcx->header.palette_type);
   printf("\n\n");
   getch();
}


// EOF
