/*
   PCX.H
*/


#ifndef __PCXH__
#define __PCXH__


#define NO_ERROR             0
#define FILE_ERROR           1
#define MEMORY_ERROR         2
#define TOO_MANY_ARGUMENTS   3



/* PCX FILE HEADER STRUCTURE */
typedef struct  {
   char id;                      /* manufacturer's id (always 0xa0)  */
   char ver;                     /* PCX version                      */
   char encoding;                /* compression method (always 1)    */
   char bitsPerPixel;            /* the # of bits per pixel          */
   int xMin, yMin;               /* image start                      */
   int xMax,yMax;                /* image size                       */
   int hRes;                     /* horizontal resolution            */
   int vRes;                     /* vertical reoslution              */
   char palette16[48];           /* 16 color pallette                */
                                    (not used with 256 color)        */
   char reserved;                /* not used                         */
   char colorPlanes;             /* # of color planes                */
   int bytesPerLine;             /* # of bytes per line              */
   int paletteType;              /* pallete type (grey or color)     */
   char filler[58];              /* not used                         */
   }PCX_HEADER;


/* PCX FILE STRUCTURE */
typedef struct {
   PCX_HEADER header;                 /* The header data        */
   long imageSize;                    /* the image size         */
   unsigned char filename[80];        /* PCX file name          */
   unsigned char far *image;          /* pointer to image data  */
   unsigned char palette[3*256];      /* PCX palette            */
   }PCX;


/* function prototypes */
int PcxLoad(char far *filename, PCX *pcx);
void PcxLoadImage(FILE *pcxfile, PCX *pcx);
void PcxLoadHeader(FILE *pcxfile, PCX *pcx);
void PcxLoadPalette(FILE *pcxfile, PCX *pcx);
void PcxDisplayHeader(PCX *pcx);


#endif


